@@ -1,45 +1,30 @@
+/*
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ *
+ * SPDX-License-Identifier: MPL-2.0 and ISC
+ *
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
+ */
+
 /*
  * Copyright (C) 2002 Stichting NLnet, Netherlands, stichting@nlnet.nl.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the
- * above copyright notice and this permission notice appear in all
- * copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND STICHTING NLNET
- * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
- * STICHTING NLNET BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
- * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- * USE OR PERFORMANCE OF THIS SOFTWARE.
- *
  * The development of Dynamically Loadable Zones (DLZ) for Bind 9 was
  * conceived and contributed by Rob Butler.
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the
- * above copyright notice and this permission notice appear in all
- * copies.
+ * Permission to use, copy, modify, and distribute this software for any purpose
+ * with or without fee is hereby granted, provided that the above copyright
+ * notice and this permission notice appear in all copies.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ROB BUTLER
- * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
- * ROB BUTLER BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
- * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- * USE OR PERFORMANCE OF THIS SOFTWARE.
- */
-
-/*
- * Copyright (C) 1999-2001, 2013, 2016  Internet Systems Consortium, Inc.
- * ("ISC")
- *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+ * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
@@ -190,7 +175,8 @@ create_path_helper(char *out, const char *in, config_data_t *cd) {
 				break;
 			}
 			if (strlen((char *)&tmpPtr[i + 1]) <=
-			    (unsigned int)cd->splitcnt) {
+			    (unsigned int)cd->splitcnt)
+			{
 				break;
 			}
 			i += cd->splitcnt;
@@ -401,10 +387,12 @@ process_dir(dir_t *dir, void *passback, config_data_t *cd, dlist_t *dir_list,
 					 */
 					while ((tmpPtr = strrchr(
 							tmpString,
-							cd->pathsep)) != NULL) {
+							cd->pathsep)) != NULL)
+					{
 						if ((strlen(host) +
 						     strlen(tmpPtr + 1) + 2) >
-						    DIR_NAMEMAX) {
+						    DIR_NAMEMAX)
+						{
 							continue;
 						}
 						strcat(host, tmpPtr + 1);
@@ -412,7 +400,8 @@ process_dir(dir_t *dir, void *passback, config_data_t *cd, dlist_t *dir_list,
 						tmpPtr[0] = '\0';
 					}
 					if ((strlen(host) + strlen(tmpString) +
-					     1) <= DIR_NAMEMAX) {
+					     1) <= DIR_NAMEMAX)
+					{
 						strcat(host, tmpString);
 					}
 				}
@@ -428,13 +417,15 @@ process_dir(dir_t *dir, void *passback, config_data_t *cd, dlist_t *dir_list,
 			 */
 			while (dir_read(dir) == ISC_R_SUCCESS) {
 				if (strncasecmp(".host", dir->entry.name, 5) ==
-				    0) {
+				    0)
+				{
 					/*
 					 * handle filesystem's special
 					 * wildcard "-"
 					 */
 					if (strcmp((char *)&dir->entry.name[6],
-						   "-") == 0) {
+						   "-") == 0)
+					{
 						strcpy(host, "*");
 					} else {
 						strncpy(host,
